const q_builder = (db_name, username, pass = '', host = 'localhost', port = 3306) => {
    const mysql = require('mysql');
    let query = '';
    const selection = [];
    const whereClause = [];
    const joins = [];
    const orders = {};
    const groups = [];
    let tableFrom = "";
    let isCon = false;

    const connection = mysql.createConnection({
        host: host,
        port: port,
        database: db_name,
        user: username,
        password: pass
    });

    this.select = (select = '*') => {
        selection.push(select);
        return this;
    }

    this.from = (table) => {
        tableFrom = table;
        return this;
    }

    this.where = (column, value, operator = '=') => {
        if (whereClause.length == 0)
            whereClause.push({ col: column, value: value, operator: operator, tipe: ' ' });
        else
            whereClause.push({ col: column, value: value, operator: operator, tipe: 'AND' });
        return this;
    }

    this.or_where = (column, value, operator = '=') => {
        whereClause.push({ col: column, value: value, operator: operator, tipe: 'OR' });
        return;
    }

    this.join = (table, on, tipe = 'INNER') => {
        joins.push({ table: table, on: on, tipe: tipe });
        return this;
    }

    this.order_by = (column, order = 'ASC') => {
        orders = { col: column, order: order };
        return this;
    }

    this.group_by = (column) => {
        groups.push(column);
        return this;
    }

    this.row = (callback, table = null) => {
        if (table != null)
            tableFrom = table;
        beforeExecute();
        connection.query(build_query(), callback)
    }

    this.results = (callback, table = null) => {
        if (table != null)
            tableFrom = table;

        if(callback == null)
            callback = () =>{};
        beforeExecute();
        connection.query(build_query(), callback);
        connection.end();
    }

    this.insert = (tabel, input, callback = null) => {
        let query = 'INSERT INTO';
        if(Array.isArray(input)){
            input.forEach((v, i) => {
                if(i == 0){
                    query += ` ${tabel} (${Object.keys(v).join(', ')}) VALUES (${Object.keys(v).join(', ')})`
                }else{
                    query += `,(${Object.keys(v).join(', ')})`;
                }
            })
        }else{
            query = `INSERT INTO ${tabel} (${Object.keys(input).join(', ')}) VALUES (${Object.values(input).join(', ')})`;
        }

        if(callback == null)
            callback = () =>{};
        beforeExecute();
        connection.query(query, callback);
        connection.end();
    }

    this.update = (table, input, callback = null) => {
        let query = `UPDATE ${table} SET`;
        Object.keys(input).forEach((k, i) => {
            if(i == 0)
                query += ` ${k} = ${input[k]}`;
            else
                query += `, ${k} = ${input[k]}`;
        });

        if(callback == null)
            callback = () =>{};
        beforeExecute();
        connection.query(build_query(query), callback);
        connection.end();
    }

    this.delete = (table, callback = null) => {
        let query = `DELETE FROM ${table}`;

        if(callback == null)
            callback = () =>{};
        beforeExecute();
        connection.query(build_query(query), callback);
        connection.end();
    }
    const beforeExecute = () => {
        if(!isCon){
            connection.connect(err => {
                if(err){
                   throw console.log(err);
                }
                isCon = true;
            });
        }

    }

    const build_query = (query = 'select') => {
        let compiled_query = ``;

        if(query == 'select')
            compiled_query = `SELECT ${selection.join(', ')} FROM ${tableFrom}`;
        else
            compiled_query = query + ' ';

        if(joins.length > 0){
            joins.forEach(j => {
                compiled_query += ` ${j.tipe} ${$.table} ${j.on}`;
            });
        } 

        whereClause.forEach((w, i) => {
            if (i == 0)
                compiled_query += ` WHERE ${w.col} ${w.operator} ${w.value} `;
            else
                compiled_query += ` ${w.tipe} ${w.col} ${w.operator} ${w.value} `;

        });

        if (groups.length != 0)
            compiled_query += ` GROUP By ${groups.join(", ")}`;

        if (Object.keys(orders).length != 0)
            compiled_query += ` ORDER BY ${orders.col} ${orders.tipe}`;

        return compiled_query;
    }

    return this;
}

module.exports = q_builder;